from textual.app import App, ComposeResult
from textual.widgets.option_list import Option
from textual.widgets import *
from textual.screen import Screen, ModalScreen
from textual.containers import Grid
from textual.binding import Binding
from textual.logging import TextualHandler
from textual import events
from rich.table import Table

from toolshelf.exceptions import InvalidToolException
from toolshelf.models.tool_item import ToolItem
from toolshelf.database import session
from toolshelf.screens import ToolScreenModal, ToolDescriptionScreen, ConfirmScreenModal, EditToolScreenModal
from toolshelf.managers.tool_manager import ToolManager as tm

import pyperclip
import logging
import subprocess

logging.basicConfig(
    level="ERROR",
    handlers=[TextualHandler()],
)

class ToolShelfApp(App):

    selected_option: Option = None

    option_list = OptionList(*[Option(tm.get_tool_color(tool), id=tool.id) for tool in tm.get_tools()], classes="sidebar")

    SCREENS = {
        "toolModal": lambda: ToolScreenModal(classes="modal"),
        "confirmModal": lambda: ConfirmScreenModal(classes="modal")
    }

    
    CSS_PATH = [
        "styling/dock_sidebar.tcss",
        "styling/create_tool_modal.tcss",
        "styling/tool_description.tcss"
    ]

    BINDINGS = [
        Binding(key="q", action="quit", description="Quit", priority=True),
        Binding(key="c", action="create", description="Create"),
        Binding(key="delete", action="delete", description="Delete"),
        Binding(key="p", action="copy", description="Copy"),
        Binding(key="e", action="edit", description="Edit"),
        Binding(key="l", action="focus", description="Change panel"),
    ]

    #----------------------ACTIONS-------------------------------------------
    def action_focus(self):
        self.app.action_focus_next()

    def action_create(self):
        self.push_screen("toolModal", self.create_tool)

    def action_delete(self):
        try:
            self.push_screen("confirmModal", self.delete_tool)
        except:
            pass
    
    def action_edit(self):
        self.push_screen(EditToolScreenModal(tool=tm.get_tool(self.selected_option.option_id), classes="modal"), self.edit_tool)

    def action_copy(self):
        toolItem: ToolItem = tm.get_tool(self.selected_option.option_id)
        pyperclip.copy(toolItem.command)

    #--------------------------METHODS----------------------------------------
    def create_tool(self, tool: ToolItem):
        try:
            tm.add_tool(tool)
            self.option_list.add_option(Option(tm.get_tool_color(tool), id=tool.id))
        except InvalidToolException as e:
            self.notify("Invalid tool!", severity="error")
            
    
    def edit_tool(self, tool: ToolItem):
        tm.edit_tool(toolItemId=self.selected_option.option_id, tool=tool)
        self.option_list.replace_option_prompt(self.selected_option.option_id, tm.get_tool_color(tool))
        self.query_one(ToolDescriptionScreen).toolItem = tool

    def delete_tool(self, delete: bool):
            if(delete):
                try:
                    
                    tm.delete_tool(toolItemId=self.selected_option.option_id)
                    self.option_list.remove_option(option_id=self.selected_option.option_id)
                    self.notify("Tool deleted!")
                except:
                    self.notify("Cannot delete tool!", severity="error")


    #---------------------------EVENTS------------------------------------
    def on_option_list_option_highlighted(self, option):
        self.selected_option = option
        # self.log(self.query_one(ToolDescriptionScreen).toolItem)
        
        self.log(tm.get_tool(option.option_id))
        self.query_one(ToolDescriptionScreen).toolItem = tm.get_tool(option.option_id)
        
    def on_option_list_option_selected(self, option):
        # option = OptionSelected
        with self.suspend():
            tool: ToolItem = tm.get_tool(option.option_id)
            self.log(tool.name)  
            subprocess.call([tool.command])

    #-----------------------------_COMPOSE-------------------------
    def compose(self) -> ComposeResult:
        yield self.option_list
        yield ToolDescriptionScreen(id="description")
        yield Footer()
    
app = ToolShelfApp().run()

# if __name__ == "__main__":
#     app = ToolShelfApp()
#     app.run(mouse= True)


from toolshelf.database import engine, Base
from sqlalchemy import Column, Integer, String, Boolean, select


class ToolItem(Base):
    __tablename__ = 'tools'
    
    id = Column(Integer, primary_key=True)
    name = Column(String, nullable=False)
    description = Column(String)
    command = Column(String)

    def __init__(self, name: str="", description: str="", command: str=""):
        self.name = name
        self.description = description
        self.command = command

    

# Create the table in the database
Base.metadata.create_all(engine)




from textual.screen import Screen
from textual.widget import Widget
from textual.widgets import Label, Static, TextArea
from textual.containers import Horizontal, VerticalScroll
from toolshelf.models.tool_item import ToolItem
from textual.reactive import reactive
from rich.table import Table
from toolshelf.widgets import ToolDescLabelWidget
from toolshelf.managers.tool_manager import ToolManager as tm
from textual.containers import ScrollableContainer



class ToolDescriptionScreen(ScrollableContainer):

    toolItem: ToolItem = reactive(ToolItem())
    # text_area = TextArea.code_editor(toolItem.name, language="markdown", read_only=True, show_line_numbers=False)
    
    CSS_PATH = [
        "styling/tool_description.tcss",
    ]

    def compose(self):
        yield Label("name:", classes="tooltype")
        yield Label("", id='name', classes="tooldescription")

        yield Label("description:", classes="tooltype")
        yield TextArea("empty...", id="description", read_only=True, disabled=True)

        yield Label("command:", classes="tooltype")
        yield Label("", id="command", classes="tooldescription")

        yield Label("installed:", classes="tooltype")
        yield Label("", id="installed", classes="tooldescription")




    def watch_toolItem(self, toolItem: ToolItem):
        self.get_widget_by_id(id='name').update(self.toolItem.name)
        if(self.toolItem.description is not ""):
            self.get_widget_by_id(id='description').text = self.toolItem.description
        else:
            self.get_widget_by_id(id='description').text = "empty"
        self.get_widget_by_id(id="command").update(self.toolItem.command)

        if(tm.get_installed(toolItem)):
            self.get_widget_by_id(id='installed').update(str("✅"))
        else:
            self.get_widget_by_id(id='installed').update(str("❌"))








from textual.screen import ModalScreen, Screen
from textual.widgets import Label, Footer, OptionList, Button
from textual.containers import Grid
from textual.binding import Binding


class ConfirmScreenModal(ModalScreen[bool]):

    BINDINGS = [
        Binding(key="ctrl+q", action="quit", description="Exit screen"),
        Binding(key="right", action="right", description="right", show=False),
        Binding(key="left", action="left", description="left", show=False)

    ]

    def action_right(self):
        self.app.action_focus_next()
        
    def action_left(self):
        self.app.action_focus_previous()

    def on_button_pressed(self, event: Button.Pressed):
        if(event.button.id == "yes"):
            self.dismiss(True)
        else:
            self.dismiss(False)

    def action_quit(self):
        self.dismiss(False)

    def compose(self):
        yield Grid(
                Label("Are you sure you want to delete?", id="question"),
                Grid(
                Button("yes", variant="success", id="yes"),
                Button("no", variant="error", id="no"),
                    classes="button-grid"
                ),
                id="dialog",
            )
        yield Footer()


from .create_tool_modal import ToolScreenModal
from .tool_description import ToolDescriptionScreen
from .confirm_modal import ConfirmScreenModal
from .edit_tool_modal import EditToolScreenModal

__all__ = ['ToolScreenModal', 'ToolDescriptionScreen']


from textual.widget import Widget
from textual.widgets import Static, Label


class ToolDescLabelWidget(Static):

    type: str
    desc: str

    def __init__(self, type, desc):
        self.type = type
        self.desc = desc
        super().__init__()

    def compose(self):
        yield Label(self.type, id="type")
        yield Label(self.desc, id="description")



from textual.app import App
from textual.widgets import *
from textual.screen import Screen, ModalScreen
from textual.containers import Grid
from textual.binding import Binding
from textual.widgets.option_list import Option
from textual.widgets import *
from toolshelf.models.tool_item import ToolItem


class ToolScreenModal(ModalScreen[ToolItem]):

    BINDINGS = [
        Binding(key="ctrl+q", action="quit", description="Exit screen"),
        Binding(key="ctrl+s", action="create", description="Create tool"),

    ]

    INPUTS: list[Input] = [
        Input(placeholder="Tool name", max_length=15, id="name"),
        Input(placeholder="Description", max_length=100, id="description"),
        Input(placeholder="Command", max_length=50, id="command")
    ]

    def reset_state(self):
        for input_field in self.INPUTS:
            input_field.value = ""
        self.INPUTS[0].focus()

    def action_quit(self):
        self.reset_state()
        self.dismiss()

    def action_create(self):
        tool = ToolItem(self.INPUTS[0].value, self.INPUTS[1].value, self.INPUTS[2].value)
        # add_tool(tool)
        # option_list.add_option(Option(tool.name))
        self.reset_state()
        self.dismiss(tool)

    # def action_cursor_down(self):
    #     if(self.cursor <= len(self.INPUTS)):
    #         self.cursor += 1
    #         self.INPUTS[self.cursor].focus()
    
    # def action_cursor_up(self):
    #     if(self.cursor >= 0):
    #         self.cursor -= 1
    #         self.INPUTS[self.cursor].focus()
        

    def compose(self):
    
        yield Grid(
                Label("Add a new tool", id="question"),
                *self.INPUTS,
                id="dialog",
            )
        yield Footer()


from toolshelf.exceptions import InvalidToolException
from toolshelf.models.tool_item import ToolItem
from sqlalchemy import Column, Integer, String, Boolean, select, update
from toolshelf.database import session
import subprocess

class ToolManager:
    @staticmethod
    def add_tool(toolItem: "ToolItem"):

        if len(toolItem.name) >= 1:
            session.add(toolItem)
            session.commit()
        else:
            raise InvalidToolException("Tool name can't be empty!")


    @staticmethod
    def delete_tool(toolItemId: int) -> None:
        session.delete(ToolManager.get_tool(toolItemId))
        session.commit()

    @staticmethod
    def edit_tool(toolItemId: int, tool: ToolItem):
        session.execute(
            update(ToolItem).where(ToolItem.id == toolItemId)
                .values(
                    name= tool.name,
                    description = tool.description,
                    command = tool.command,
                )
        )
        session.commit()
    

    @staticmethod
    def get_tools():
        toolItems = session.query(ToolItem).all()

        toolItems_sorted = sorted(toolItems, key=lambda x: x.name, reverse=False)
        return toolItems_sorted
    

    @staticmethod
    def get_tool(toolItemId: int) -> "ToolItem":
        statement = select(ToolItem).filter_by(id=toolItemId)
        
        return session.get_one(ToolItem, toolItemId)

    @staticmethod
    def get_installed(tool: "ToolItem") -> bool:
        try:
            result = subprocess.run(['which', tool.command], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            return result.returncode == 0
        except FileNotFoundError:
            return False
        
    @staticmethod
    def get_tool_color(tool: "ToolItem") -> str:
        if ToolManager.get_installed(tool):
            return "[green]" + tool.name
        else:
            return "[red]" + tool.name




from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
import os

share_dir = os.path.expanduser('~/.local/share/toolshelf')
db_path = os.path.join(share_dir, 'toolshelf.db')
os.makedirs(share_dir, exist_ok=True)
DATABASE_URL = f'sqlite:///{db_path}'

# Create a new SQLite database (or connect to an existing one)
engine = create_engine(DATABASE_URL, echo=False)

# Base class for declarative class definitions
Base = declarative_base()

# Create a configured "Session" class
Session = sessionmaker(bind=engine, autoflush=True)

# Create a session
session = Session()




from setuptools import setup, find_packages

# read the contents of README file
from pathlib import Path
this_directory = Path(__file__).parent

VERSION = '0.0.5' 
DESCRIPTION = 'A versatile tool management utility for developers'
LONG_DESCRIPTION = (this_directory / "README.md").read_text()

# Setting up
setup(
    name="toolshelf-ui", 
    version=VERSION,
    author="Atophite",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    url="https://github.com/Atophite/toolshelf",
    packages=find_packages(),
    install_requires=["textual", "rich", "sqlalchemy", "pyperclip"], 
    include_package_data=True,
    package_data={
        "toolshelf": ["styling/*.tcss"],
    },
    keywords=['python', 'command-line', 'tool', 'TUI', 'Terminal User Interface'],
    classifiers= [
        "Programming Language :: Python :: 3",
    ],
    py_modules=["toolshelf"],
    entry_points={
        'console_scripts': [
            'toolshelf=toolshelf.entry_points.toolshelf_runner:run_toolshelf',
        ],
    }

)


from textual.app import App
from textual.widgets import *
from textual.screen import Screen, ModalScreen
from textual.containers import Grid
from textual.binding import Binding
from textual.widgets.option_list import Option
from textual.widgets import *
from toolshelf.models.tool_item import ToolItem
from toolshelf.managers.tool_manager import ToolManager as tm


class EditToolScreenModal(ModalScreen[ToolItem]):

    BINDINGS = [
        Binding(key="ctrl+q", action="quit", description="Exit screen"),
        Binding(key="ctrl+s", action="edit", description="Edit tool"),

    ]

    INPUTS: list[Input] = [
        Input(placeholder="Tool name", max_length=15, id="name"),
        Input(placeholder="Description", max_length=100, id="description"),
        Input(placeholder="Command", max_length=50, id="command")
    ]

    def __init__(self, tool: ToolItem, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.tool = tool
        self.INPUTS: list[Input] = [
            Input(value=tool.name, placeholder="Tool name", max_length=15, id="name"),
            Input(value=tool.description, placeholder="Description", max_length=100, id="description"),
            Input(value=tool.command, placeholder="Command", max_length=50, id="command")
        ]

    def reset_state(self):
        for input_field in self.INPUTS:
            input_field.value = ""
        self.INPUTS[0].focus()

    def action_quit(self):
        self.dismiss()

    def action_edit(self):
        tool = ToolItem(self.INPUTS[0].value, self.INPUTS[1].value, self.INPUTS[2].value)

        self.dismiss(tool)
        

    def compose(self):
    
        yield Grid(
                Label(f"Edit tool {self.tool.name}", id="question"),
                *self.INPUTS,
                id="dialog",
            )
        yield Footer()